@@ -1,4 +1,4 @@
-/* Copyright © 2007, 2008, 2009, 2010 Jakub Wilk
+/* Copyright © 2007-2014 Jakub Wilk
  * Copyright © 2009 Mateusz Turcza
  *
  * This package is free software; you can redistribute it and/or modify
@@ -14,12 +14,13 @@
 
 #include <getopt.h>
 
+#include "autoconf.hh"
+
 #include "config.hh"
 
 #include "debug.hh"
-#include "djvuconst.hh"
+#include "djvu-const.hh"
 #include "i18n.hh"
-#include "version.hh"
 
 class OptionsParseError : public Config::Error
 {
