@@ -1,4 +1,4 @@
-/* Copyright © 2007, 2008, 2009, 2010 Jakub Wilk
+/* Copyright © 2007-2015 Jakub Wilk
  * Copyright © 2009 Mateusz Turcza
  *
  * This package is free software; you can redistribute it and/or modify
@@ -21,18 +21,21 @@
 #include <omp.h>
 #endif
 
-#include "pdf-backend.hh"
-#include "pdf-dpi.hh"
+#include "autoconf.hh"
+
 #include "config.hh"
 #include "debug.hh"
-#include "djvuconst.hh"
-#include "paths.hh"
+#include "djvu-const.hh"
+#include "djvu-outline.hh"
+#include "i18n.hh"
 #include "image-filter.hh"
+#include "paths.hh"
+#include "pdf-backend.hh"
+#include "pdf-dpi.hh"
 #include "sexpr.hh"
 #include "system.hh"
-#include "version.hh"
-#include "i18n.hh"
 #include "xmp.hh"
+#include "version.hh"
 
 Config config;
 
@@ -656,14 +659,19 @@ public:
   { }
 };
 
-static sexpr::Expr pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *catalog,
-  const ComponentList &page_files)
+static const int pdf_outline_max_depth = 0x100;
+
+void pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *catalog,
+  djvu::OutlineBase &djvu_outline, const ComponentList &page_files,
+  int depth)
 {
-  sexpr::GCLock gc_lock;
+  if (depth > pdf_outline_max_depth)
+    /* DjVu specification puts no limit on outline depth,
+     * but we want to avoid stack overflow because of very deep recursion. */
+    throw djvu::OutlineError();
   pdf::Object current, next;
   if (!pdf::dict_lookup(node, "First", &current)->isDict())
-    return sexpr::nil;
-  sexpr::Ref list = sexpr::nil;
+    return;
   while (current.isDict())
   {
     try
@@ -690,16 +698,13 @@ static sexpr::Expr pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *
           throw NoPageForBookmark();
         page = get_page_for_LinkGoTo(dynamic_cast<pdf::link::GoTo*>(link_action.get()), catalog);
       }
-      sexpr::Ref expr = sexpr::nil;
-      expr = pdf_outline_to_djvu_outline(&current, catalog, page_files);
       {
-        std::ostringstream strstream;
-        strstream << "#" << page_files.get_file_name(page);
-        sexpr::Ref pexpr = sexpr::string(strstream.str());
-        expr = sexpr::cons(pexpr, expr);
+        djvu::OutlineItem &djvu_outline_item = djvu_outline.add(
+          title_str,
+          std::string("#") + page_files.get_file_name(page)
+        );
+        pdf_outline_to_djvu_outline(&current, catalog, djvu_outline_item, page_files, depth + 1);
       }
-      expr = sexpr::cons(sexpr::string(title_str), expr);
-      list = sexpr::cons(expr, list);
     }
     catch (BookmarkError &ex)
     {
@@ -711,31 +716,21 @@ static sexpr::Expr pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *
     current = next;
   }
   current.free();
-  list.reverse();
-  return list;
 }
 
-static bool pdf_outline_to_djvu_outline(pdf::Document &doc, std::ostream &stream,
+static void pdf_outline_to_djvu_outline(pdf::Document &doc, djvu::Outline &djvu_outline,
   const ComponentList &page_files)
-/* Convert the PDF outline to DjVu outline. Save the resulting S-expression
- * into the stream.
+/* Convert the PDF outline to DjVu outline.
  *
  * Return ``true`` if the outline exist and is non-empty.
  * Return ``false`` otherwise.
  */
 {
-  sexpr::GCLock gc_lock;
   pdf::Catalog *catalog = doc.getCatalog();
-  pdf::Object *outlines = catalog->getOutline();
-  if (!outlines->isDict())
-    return false;
-  static sexpr::Ref symbol_bookmarks = sexpr::symbol("bookmarks");
-  sexpr::Ref expr = pdf_outline_to_djvu_outline(outlines, catalog, page_files);
-  if (expr == sexpr::nil)
-    return false;
-  expr = sexpr::cons(symbol_bookmarks, expr);
-  stream << expr;
-  return true;
+  pdf::Object *pdf_outline = catalog->getOutline();
+  if (!pdf_outline->isDict())
+    return;
+  pdf_outline_to_djvu_outline(pdf_outline, catalog, djvu_outline, page_files, 0);
 }
 
 static void add_meta_string(const char *key, const std::string &value, std::ostream &stream)
@@ -770,7 +765,7 @@ private:
   TemporaryComponentList& operator=(const TemporaryComponentList&); // not defined
 protected:
   std::auto_ptr<const TemporaryDirectory> directory;
-  std::auto_ptr<const TemporaryFile> shared_ant_file;
+  std::auto_ptr<TemporaryFile> shared_ant_file;
 
   virtual File *create_file(const std::string &pageid)
   {
@@ -781,7 +776,10 @@ public:
   : ComponentList(n, page_map),
     directory(new TemporaryDirectory()),
     shared_ant_file(new TemporaryFile(*directory, djvu::shared_ant_file_name))
-  { }
+  {
+    shared_ant_file->write("AT&TFORM\0\0\0\4DJVI", 16);
+    shared_ant_file->close();
+  }
 
   virtual ~TemporaryComponentList() throw ()
   {
@@ -815,20 +813,16 @@ protected:
     return DjVuCommand::dir_name + "/" + base_name;
   }
 public:
-  static void set_argv0(const char *argv0);
   explicit DjVuCommand(const std::string &base_name)
   : Command(full_path(base_name))
   { }
 };
 
-std::string DjVuCommand::dir_name("");
-
-void DjVuCommand::set_argv0(const char *argv0)
-{
-  std::string argv0_dir_name, argv0_file_name;
-  split_path(argv0, argv0_dir_name, argv0_file_name);
-  DjVuCommand::dir_name = absolute_path(paths::djvulibre_bindir, argv0_dir_name);
-}
+#if WIN32
+std::string DjVuCommand::dir_name(program_dir);
+#else
+std::string DjVuCommand::dir_name(paths::djvulibre_bindir);
+#endif
 
 class DjVm
 {
@@ -852,16 +846,15 @@ protected:
   void remember(const Component &component);
 public:
   virtual void add(const Component &compontent) = 0;
-  virtual void create() = 0;
   virtual void commit() = 0;
   DjVm &operator <<(const Component &component)
   {
     this->add(component);
     return *this;
   }
-  virtual void set_outline(File &outline_sed_file) = 0;
+  virtual void set_outline(const djvu::Outline &outline) = 0;
   virtual void set_metadata(File &metadata_sed_file) = 0;
-  virtual ~DjVm() throw () { /* just to shut up compilers */ }
+  virtual ~DjVm() throw () { /* just to silence compilers */ }
 };
 
 void DjVm::remember(const Component &component)
@@ -885,21 +878,22 @@ class IndirectDjVm;
 class BundledDjVm : public DjVm
 {
 protected:
+  size_t size;
   File &output_file;
   DjVuCommand converter;
   std::auto_ptr<IndirectDjVm> indirect_djvm;
   std::auto_ptr<TemporaryFile> index_file;
 public:
   explicit BundledDjVm(File &output_file)
-  : output_file(output_file),
+  : size(0),
+    output_file(output_file),
     converter("djvmcvt")
   { }
   ~BundledDjVm() throw ()
   { }
   virtual void add(const Component &component);
-  virtual void set_outline(File &outlines_sed_file);
+  virtual void set_outline(const djvu::Outline &outline);
   virtual void set_metadata(File &metadata_sed_file);
-  virtual void create();
   virtual void commit();
 };
 
@@ -908,6 +902,8 @@ class IndirectDjVm : public DjVm
 protected:
   File &index_file;
   std::vector<Component> components;
+  bool needs_shared_ant;
+  std::auto_ptr<std::ostringstream> outline_stream;
   class UnexpectedDjvuSedOutput : public std::runtime_error
   {
   public:
@@ -915,10 +911,12 @@ protected:
     : std::runtime_error(_("Unexpected output from djvused"))
     { };
   };
-  void do_create(const std::vector<Component> &components, bool shared_ant = false);
+  void create_bare(const std::vector<Component> &components);
+  void create(const std::vector<Component> &components, bool bare=false);
 public:
   explicit IndirectDjVm(File &index_file)
-  : index_file(index_file)
+  : index_file(index_file),
+    needs_shared_ant(false)
   { }
 
   virtual ~IndirectDjVm() throw ()
@@ -930,33 +928,28 @@ public:
     this->components.push_back(component);
   }
 
-  virtual void set_outline(File &outlines_sed_file)
+  virtual void set_outline(const djvu::Outline &outline)
   {
-    debug(3) << _("creating document outline with `djvused`") << std::endl;
-    DjVuCommand djvused("djvused");
-    djvused << "-s" << "-f" << outlines_sed_file << this->index_file;
-    djvused(); // djvused -s -f <outlines-sed-file> <index-djvu-file>
+    if (!outline)
+    {
+      this->outline_stream.reset(NULL);
+      return;
+    }
+    this->outline_stream.reset(new std::ostringstream);
+    *this->outline_stream << outline;
   }
 
   virtual void set_metadata(File &metadata_sed_file)
   {
     size_t size = this->components.size();
-    if (size <= 2)
-    {
-      debug(3) << _("setting metadata with `djvused`") << std::endl;
-      DjVuCommand djvused("djvused");
-      djvused << "-s" << "-f" << metadata_sed_file << this->index_file;
-      djvused(); // djvused -s -f <metadata-sed-file> <index-djvu-file>
-    }
-    else
     {
       /* Using ``djvused`` to add shared annotations to a indirect multi-page
        * document could be unacceptably slow:
-       * http://sf.net/tracker/?func=detail&aid=1935916&group_id=32953&atid=406583
+       * http://sourceforge.net/p/djvu/bugs/100/
        *
        * We need to work around this bug.
        */
-      debug(3) << _("setting metadata with `djvused` (working around a DjVuLibre bug)") << std::endl;
+      debug(3) << _("setting metadata with `djvused`") << std::endl;
       std::vector<Component> dummy_components;
       TemporaryFile dummy_sed_file;
       /* For an indirect document, ``create-shared-ant`` is, surprisingly, not
@@ -977,7 +970,7 @@ public:
          * - Add the dummy shared annotation.
          */
         dummy_components.push_back(this->components[i]);
-        this->do_create(dummy_components);
+        this->create_bare(dummy_components);
         DjVuCommand djvused("djvused");
         djvused << "-s" << "-f" << dummy_sed_file << this->index_file;
         djvused(); // djvused -s -f <dummy-sed-file> <output-djvu-file>
@@ -985,30 +978,32 @@ public:
       }
       /* For the last page, use the desired annotations. */
       dummy_components.push_back(this->components[size - 1]);
-      this->do_create(dummy_components);
+      this->create_bare(dummy_components);
       DjVuCommand djvused("djvused");
       djvused << "-s" << "-f" << metadata_sed_file << this->index_file;
       djvused(); // djvused -s -f <metadata-sed-file> <output-djvu-file>
-      /* Finally, recreate the index. */
-      this->do_create(this->components, true /* include shared annotations */);
+      this->needs_shared_ant = true;
     }
   }
 
-  virtual void create()
+  virtual void commit()
   {
     size_t size = this->components.size();
     debug(3)
       << string_printf(ngettext(
-           "creating multi-page indirect document (%d page)",
-           "creating multi-page indirect document (%d pages)",
+           "creating multi-page indirect document (%zu page)",
+           "creating multi-page indirect document (%zu pages)",
            size), size
          )
       << std::endl;
-    this->do_create(this->components);
+    this->create(this->components);
+  }
+
+  void require_shared_ant()
+  {
+    this->needs_shared_ant = true;
   }
 
-  virtual void commit()
-  { }
 };
 
 void BundledDjVm::add(const Component &component)
@@ -1022,11 +1017,17 @@ void BundledDjVm::add(const Component &component)
   }
   /* No need to call ``this->remember(component)``. */
   *this->indirect_djvm << component;
+  this->size++;
 }
 
-void BundledDjVm::set_outline(File &outlines_sed_file)
+void BundledDjVm::set_outline(const djvu::Outline &outline)
 {
-  this->indirect_djvm->set_outline(outlines_sed_file);
+  this->indirect_djvm->set_outline(outline);
+  if (this->size < 2)
+    /* Some old DjVuLibre versions (at least 3.5.23) don't preserve outline in
+     * single-page documents without shared annotation chunk. Let's work around
+     * this problem. */
+    this->indirect_djvm->require_shared_ant();
 }
 
 void BundledDjVm::set_metadata(File &metadata_sed_file)
@@ -1034,13 +1035,9 @@ void BundledDjVm::set_metadata(File &metadata_sed_file)
   this->indirect_djvm->set_metadata(metadata_sed_file);
 }
 
-void BundledDjVm::create()
-{
-  this->indirect_djvm->create();
-}
-
 void BundledDjVm::commit()
 {
+  this->indirect_djvm->commit();
   this->converter
     << "-b"
     << *this->index_file
@@ -1049,12 +1046,17 @@ void BundledDjVm::commit()
   this->index_file.reset(NULL);
 }
 
-void IndirectDjVm::do_create(const std::vector<Component> &components, bool shared_ant)
+void IndirectDjVm::create_bare(const std::vector<Component> &components)
+{
+  this->create(components, true);
+}
+
+void IndirectDjVm::create(const std::vector<Component> &components, bool bare)
 {
   size_t size = components.size();
   this->index_file.reopen(true); // (re)open and truncate
-  this->index_file.write(djvu::binary::djvm_head, sizeof djvu::binary::djvm_head);
-  this->index_file << djvu::binary::version;
+  this->index_file.write("AT&TFORM\0\0\0\0DJVMDIRM\0\0\0\0\1", 25);
+  bool shared_ant = !bare && this->needs_shared_ant;
   for (int i = 1; i >= 0; i--)
     index_file << static_cast<char>(((size + shared_ant) >> (8 * i)) & 0xff);
   {
@@ -1079,13 +1081,29 @@ void IndirectDjVm::do_create(const std::vector<Component> &components, bool shar
     bzz << "-e" << bzz_file << "-";
     bzz(index_file);
   }
-  size = this->index_file.size();
-  this->index_file.seekg(8, std::ios::beg);
-  for (int i = 3; i >= 0; i--)
-    this->index_file << static_cast<char>(((size - 12) >> (8 * i)) & 0xff);
+  size_t dirm_off = this->index_file.size();
   this->index_file.seekg(20, std::ios::beg);
   for (int i = 3; i >= 0; i--)
-    this->index_file << static_cast<char>(((size - 24) >> (8 * i)) & 0xff);
+    this->index_file << static_cast<char>(((dirm_off - 24) >> (8 * i)) & 0xff);
+  if (!bare && this->outline_stream.get())
+  {
+    TemporaryFile bzz_file;
+    this->index_file.seekg(0, std::ios::end);
+    index_file.write("NAVM\0\0\0", 8);
+    bzz_file << this->outline_stream->str();
+    bzz_file.close();
+    DjVuCommand bzz("bzz");
+    bzz << "-e" << bzz_file << "-";
+    bzz(index_file);
+    size_t outline_off = index_file.size();
+    this->index_file.seekg(dirm_off + 4, std::ios::beg);
+    for (int i = 3; i >= 0; i--)
+      this->index_file << static_cast<char>(((outline_off - dirm_off - 8) >> (8 * i)) & 0xff);
+  }
+  size_t off = this->index_file.size();
+  this->index_file.seekg(8, std::ios::beg);
+  for (int i = 3; i >= 0; i--)
+    this->index_file << static_cast<char>(((off - 12) >> (8 * i)) & 0xff);
   this->index_file.close();
 }
 
@@ -1165,9 +1183,7 @@ static void calculate_subsampled_size(int width, int height, int ratio, int &sub
    * This functions satisfies all these equations by decreasing ratio if
    * necessary.
    *
-   * See
-   * http://sf.net/tracker/?func=detail&aid=1972089&group_id=32953&atid=406583
-   * for details.
+   * See http://sourceforge.net/p/djvu/bugs/106/ for details.
    */
   while (true)
   {
@@ -1183,7 +1199,6 @@ static void calculate_subsampled_size(int width, int height, int ratio, int &sub
 static int xmain(int argc, char * const argv[])
 {
   std::ios_base::sync_with_stdio(false);
-  DjVuCommand::set_argv0(argv[0]);
 
   try
   {
@@ -1191,7 +1206,7 @@ static int xmain(int argc, char * const argv[])
   }
   catch (const Config::NeedVersion)
   {
-    error_log << PACKAGE_STRING << std::endl;
+    error_log << get_multiline_version();
     exit(1);
   }
   catch (const Config::Error &ex)
@@ -1202,10 +1217,14 @@ static int xmain(int argc, char * const argv[])
     exit(1);
   }
 
-  if (config.output_stdout && is_stream_a_tty(std::cout))
-    throw StdoutIsATerminal();
+  if (config.output_stdout)
+  {
+    if (isatty(std::cout))
+      throw StdoutIsATerminal();
+    binmode(std::cout);
+  }
 
-  pdf::Environment environment(argv[0]);
+  pdf::Environment environment;
   environment.set_antialias(config.antialias);
 
   DocumentMap document_map(config.filenames);
@@ -1224,6 +1243,7 @@ static int xmain(int argc, char * const argv[])
   std::auto_ptr<DjVm> djvm;
   std::auto_ptr<ComponentList> page_files;
   std::auto_ptr<Quantizer> quantizer;
+  djvu::Outline djvu_outline;
   if (config.monochrome)
     quantizer.reset(new DummyQuantizer(config));
   else
@@ -1596,7 +1616,6 @@ static int xmain(int argc, char * const argv[])
     int n = page_numbers[i];
     *djvm << (*page_files)[n];
   }
-  djvm->create();
   /* Only first PDF document metadata/outline is taken into account. */
   doc.reset(new pdf::Document(config.filenames[0]));
   if (config.extract_metadata)
@@ -1640,21 +1659,9 @@ static int xmain(int argc, char * const argv[])
   }
   if (config.extract_outline)
   {
-    bool nonempty_outline;
-    TemporaryFile sed_file;
     debug(3) << _("extracting document outline") << std::endl;
-    if (config.format == config.FORMAT_BUNDLED)
-    {
-      /* Shared annotations chunk in necessary to preserve multi-file document
-       * structure. (Single-file documents cannot contain document outline.) */
-      sed_file << "create-shared-ant" << std::endl;
-    }
-    sed_file << "set-outline" << std::endl;
-    nonempty_outline = pdf_outline_to_djvu_outline(*doc, sed_file, *page_files);
-    sed_file << std::endl << "." << std::endl;
-    sed_file.close();
-    if (nonempty_outline)
-      djvm->set_outline(sed_file);
+    pdf_outline_to_djvu_outline(*doc, djvu_outline, *page_files);
+    djvm->set_outline(djvu_outline);
   }
   djvm->commit();
   {
@@ -1664,8 +1671,8 @@ static int xmain(int argc, char * const argv[])
       djvu_size += djvu_pages_size;
       try
       {
-        ExistingFile shared_anno(*output_dir, djvu::shared_ant_file_name);
-        djvu_size += shared_anno.size();
+        ExistingFile shared_ant(*output_dir, djvu::shared_ant_file_name);
+        djvu_size += shared_ant.size();
       }
       catch (std::ios_base::failure &ex)
       {
@@ -1697,7 +1704,7 @@ static int xmain(int argc, char * const argv[])
 int main(int argc, char * const argv[])
 try
 {
-  i18n::setup(argv[0]);
+  i18n::setup();
   xmain(argc, argv);
 }
 /* Please keep the exception handlers in sync with the ones in xmain(). */
