@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2012 Jakub Wilk
+# Copyright © 2009-2015 Jakub Wilk
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ from common import (
 
 class test(case):
     '''
-    https://code.google.com/p/pdf2djvu/issues/detail?id=12
+    https://bitbucket.org/jwilk/pdf2djvu/issue/12
     fixed in [490a08283fb4]
     '''
     def test_verbatim(self):
@@ -24,6 +24,7 @@ class test(case):
         assert_grep(xmp, '>application/pdf<')
 
     def test_no_verbatim(self):
+        self.require_feature('GNOME XSLT')
         self.pdf2djvu().assert_()
         xmp = self.extract_xmp()
         assert_well_formed_xml(xmp)
